Simple logging module, that supports terminal coloring and different plugins

# Getting Started

``` javascript

const Logging = require("@hibas123/logging").Logging;

Logging.log("Hello there");

```

There are different Logging levels, that also apply terminal coloring:

``` javascript
Logging.debug("Debug message")
Logging.log("Log message")
Logging.warning("Warning")
Logging.error(new Error("To less creativity"))
Logging.error("Just an simple message as error")
Logging.errorMessage("Nearly the same as error")
```

All Logging types except the simple error take as many arguments as you want. These will be joined with spaces and serialized with the node util.inspect function.

# Setup

This logging module doesn't require any setup per default, but sometimes it makes sense to configure a few things.

For example can you disable the console output. This may be helpful, if you insert a custom adapters as described below.

Also you can set a name. All messages that are send with this instance are prefixed by this name.

``` javascript
   const CustomLogging = new LoggingBase(name | {
      name: "custom", // default undefined
      console: false // default true
   });
```


# Plugins

There is a Plugin API available, that makes is possible to add custom Logging Adapter.

``` javascript
const Demo = new LoggingBase("Demo");
Demo.addAdapter(new DemoAdapter({ color: "rainbow" }));
```

The adapters need to provide a very simple Interface:

``` typescript
interface Adapter {
    init(observable: ObservableInterface<Message>, name?: string): void | Promise<void>;

    flush(sync: true): void;
    flush(sync: false): void | Promise<void>;
}

interface Message {
    type: LoggingTypes;
    name?:string;
    text: {
    	raw: string[],
    	formatted: string[]
    };
    date: Date;
    file: string;
}

enum LoggingTypes {
   Log,
   Warning,
   Error,
   Debug
}
```

The `ObservableInterface` comes from `@hibas123/utils`. It provides a very simple api for subscribing and unsubscribing from the message events.

More Details on Observable [git](https://git.stamm.me/OpenServer/Utils) or [npm](https://www.npmjs.com/package/@hibas123/utils)

# License
MIT

Copyright (c) 2018 Fabian Stamm

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.